
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A checkbox, in combination with LcarsCheckboxGroup also usable as radiobutton.
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsCheckbox extends Canvas implements Serializable, MouseListener
{
 /**
  * Label is shown horizontally and vertically centered.
  */
 public static final int lpCenter = 0;
 /**
  * Label is shown in the upper left corner of the control.
  */
 public static final int lpTopLeft = 1;
  /**
  * Label is shown in the upper right corner of the control.
  */
 public static final int lpTopRight = 2;
 /**
  * Label is shown in the lower right corner of the control.
  */
 public static final int lpBottomRight = 3;
 /**
  * Label is shown in the lower left corner of the control.
  */
 public static final int lpBottomLeft = 4;
 /**
  * The part of the control with the label on it is at the left side.
  */
 public static final int haLeftJustify = 0;
 /**
  * The part of the control with the label on it is at the right side.
  */
 public static final int haRightJustify = 1;
 /**
  * First style: Depending on the state property the color changes
  * between checkedColor and uncheckedColor.
  */
 public static final int cbsOne = 0;
 /**
  * Second style: The control is drawn using checkedColor, but
  * when state is false the small part of the control is not drawn.
  */
 public static final int cbsTwo = 1;

 private Image backImage;
 private String label;
 private Color fontColor;
 private Color checkedColor;
 private Color uncheckedColor;
 private Color disabledColor;
 private boolean state;
 private int style;
 private int horizontalAlignment;
 private int labelPosition;
 private int labelMarginX;
 private int labelMarginY;
 private int labelPosX;
 private int labelPosY;
 private boolean roundShort;
 private boolean roundLong;
 private int radius;
 private int horizontalSpace;
 private int rectWidth;
 private Vector listeners = new Vector();

 /**
  * The LcarsCheckboxGroup which is associated with the checkbox.
  */
 protected LcarsCheckboxGroup group;

  /**
  * Constructs a new instance.
  */
 public LcarsCheckbox()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  horizontalSpace = 5;
  roundLong = true;
  roundShort = true;
  labelMarginX = 5;
  labelMarginY = 2;
  labelPosition = lpCenter;
  label = "";
  checkedColor = Color.green;
  uncheckedColor = Color.blue;
  disabledColor = Color.gray;
  fontColor = Color.black;
  horizontalAlignment = haLeftJustify;
  style = cbsOne;
  setSize(80, 25);
  addMouseListener(this);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {}

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  Color foreColor;
  Color backColor = getBackground();
  int width = getSize().width;
  int height = getSize().height;
  radius = height / 2;
  rectWidth = width - (radius * 2) - horizontalSpace;
  if (!isEnabled()) foreColor = getDisabledColor();
  else
  {
   if (state == true) foreColor = getCheckedColor();
   else
   {
    if (style == cbsOne) foreColor = getUncheckedColor();
    else foreColor = getCheckedColor();
   }
  }
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.clearRect(0, 0, width, height);
  backBuffer.setColor(foreColor);
  if (horizontalAlignment == haLeftJustify)
  {
   backBuffer.fillRect(radius, 0, width - height - horizontalSpace, height);
   if (roundLong == true)
    backBuffer.fillArc(0, 0, height, height, 90, 180);
   else if (roundLong == false)
    backBuffer.fillRect(0, 0, radius, height);
   if (!((style == cbsTwo) && (state == false)))
   {
    if (roundShort == true)
     backBuffer.fillArc(width - height, 0, height, height, 270, 180);
    else if (roundShort == false)
     backBuffer.fillRect(width - radius, 0, radius, height);
   }
  }
  else if (horizontalAlignment == haRightJustify)
  {
   backBuffer.fillRect(radius + horizontalSpace, 0, width - height - horizontalSpace, height);
   if (roundLong == true)
    backBuffer.fillArc(width - height, 0, height, height, 270, 180);
   else if (roundLong == false)
    backBuffer.fillRect(width - radius - 1, 0, radius, height);
   if (!((style == cbsTwo) && (state == false)))
   {
    if (roundShort == true)
     backBuffer.fillArc(0, 0, height, height, 90, 180);
    else if (roundShort == false)
     backBuffer.fillRect(0, 0, radius, height);
   }
  }
  backBuffer.setColor(fontColor);
  FontMetrics fm = backBuffer.getFontMetrics();
  switch (labelPosition)
  {
   case lpCenter :
    if (horizontalAlignment == haRightJustify)
    {
     if (roundLong == true)
      labelPosX = radius + horizontalSpace + ((rectWidth - fm.stringWidth(label)) / 2);
     else
      labelPosX = radius + horizontalSpace + ((rectWidth + radius - fm.stringWidth(label)) / 2);
    }
    else if (horizontalAlignment == haLeftJustify)
    {
     if (roundLong == true)
      labelPosX = radius + ((rectWidth - fm.stringWidth(label)) / 2);
     else
      labelPosX = (rectWidth + radius - fm.stringWidth(label)) / 2;
    }
    labelPosY = (height + fm.getAscent() - fm.getDescent()) / 2;
    break;
   case lpTopLeft :
    if (horizontalAlignment == haRightJustify)
     labelPosX = radius + horizontalSpace + labelMarginX;
    else if (horizontalAlignment == haLeftJustify)
    {
     if (roundLong == true)
      labelPosX = radius + labelMarginX;
     else
      labelPosX = labelMarginX;
    }
    labelPosY = labelMarginY + fm.getAscent();
    break;
   case lpTopRight :
    if (horizontalAlignment == haRightJustify)
    {
     if (roundLong == true)
      labelPosX = width - radius - labelMarginX - fm.stringWidth(label);
     else
      labelPosX = width - labelMarginX - fm.stringWidth(label);
    }
    else if (horizontalAlignment == haLeftJustify)
     labelPosX = width - radius - horizontalSpace - labelMarginX - fm.stringWidth(label);
    labelPosY = labelMarginY + fm.getAscent();
    break;
   case lpBottomRight :
    if (horizontalAlignment == haRightJustify)
    {
     if (roundLong == true)
      labelPosX = width - radius - labelMarginX - fm.stringWidth(label);
     else
      labelPosX = width - labelMarginX - fm.stringWidth(label);
    }
    else if (horizontalAlignment == haLeftJustify)
     labelPosX = width - radius - horizontalSpace - labelMarginX - fm.stringWidth(label);
    labelPosY = height - labelMarginY - fm.getDescent();
    break;
   case lpBottomLeft :
    if (horizontalAlignment == haRightJustify)
     labelPosX = radius + horizontalSpace + labelMarginX;
    else if (horizontalAlignment == haLeftJustify)
    {
     if (roundLong == true)
      labelPosX = radius + labelMarginX;
     else
      labelPosX = labelMarginX;
    }
    labelPosY = height - labelMarginY - fm.getDescent();
    break;
  }
  backBuffer.drawString(label, labelPosX, labelPosY);
  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Returns the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }

 /**
  * Returns the state of the control (true = checked, false = unchecked).
  */
 public boolean isState()
 {
  return state;
 }

 /**
  * Sets the state of the control (true = checked, false = unchecked).
  */
 public void setState(boolean newState)
 {
  LcarsCheckboxGroup currentGroup = this.group;
	if (currentGroup != null)
  {
   state = newState;
	 if (newState == true)
   {
		currentGroup.setSelectedCheckbox(this);
	 }
  }
  else if (currentGroup == null)
  {
   if (state != newState)
   {
    state = !state;
   }
  }
  repaint();
 }

 /**
  * Toggles the state property between true and false.
  */
 public void Toggle()
 {
  if (group == null)
  {
   state = !state;
   repaint();
  }
 }

 /**
  * Returns the style of the control.
  */
 public int getStyle()
 {
  return style;
 }

 /**
  * Sets the style of the control.<BR>
  * Possible values: cbsOne, cbsTwo<BR>
  * If the new value is not 0 or 1 the property is set to cbsOne.
  */
 public void setStyle(int newStyle)
 {
  if (style != newStyle)
  {
   if ((newStyle < 0) || (newStyle > 1))
    style = cbsOne;
   else
    style = newStyle;
   repaint();
  }
 }

 /**
  * Returns the color used for drawing the label.
  */
 public Color getFontColor()
 {
  return fontColor;
 }

 /**
  * Sets the color used for drawing the label.
  */
 public void setFontColor(Color newFontColor)
 {
  if (!fontColor.equals(newFontColor))
  {
   fontColor = newFontColor;
   repaint();
  }
 }

 /**
  * Returns the label.
  */
 public String getLabel()
 {
  return label;
 }

 /**
  * Sets the label.
  */
 public void setLabel(String newLabel)
 {
  if (!label.equals(newLabel))
  {
   label = newLabel;
   repaint();
  }
 }

 /**
  * Returns the position of the label.
  */
 public int getLabelPosition()
 {
  return labelPosition;
 }

 /**
  * Sets the position of the label.<BR>
  * Possible values: lpCenter, lpTopLeft, lpTopRight, lpBottomRight, lpBottomLeft<BR>
  * If the new value is not in the range between 0 and 4 the property is set to
  * lpCenter.
  */
 public void setLabelPosition(int newLabelPosition)
 {
  if (labelPosition != newLabelPosition)
  {
   if ((newLabelPosition < 0) || (newLabelPosition > 4))
    labelPosition = lpCenter;
   else
    labelPosition = newLabelPosition;
   repaint();
  }
 }

 /**
  * Returns the horizontal margin between the label and the bound of the control.
  */
 public int getLabelMarginX()
 {
  return labelMarginX;
 }

 /**
  * Sets the horizontal margin between the label and the bound of the control
  * if the label position is not lpCenter.
  */
 public void setLabelMarginX(int newLabelMarginX)
 {
  if (labelMarginX != newLabelMarginX)
  {
   labelMarginX = newLabelMarginX;
   repaint();
  }
 }

 /**
  * Returns the vertical margin between the label and the bound of the control.
  */
 public int getLabelMarginY()
 {
  return labelMarginY;
 }

 /**
  * Sets the vertical margin between the label and the bound of the control
  * if the label position is not lpCenter.
  */
 public void setLabelMarginY(int newLabelMarginY)
 {
  if (labelMarginY != newLabelMarginY)
  {
   labelMarginY = newLabelMarginY;
   repaint();
  }
 }

 /**
  * Returns the horizontal space between the part with the label and the small
  * part of the control.
  */
 public int getHorizontalSpace()
 {
  return horizontalSpace;
 }

 /**
  * Sets the horizontal space between the part with the label and the small
  * part of the control.
  */
 public void setHorizontalSpace(int newHorizontalSpace)
 {
  if (horizontalSpace != newHorizontalSpace)
  {
   horizontalSpace = newHorizontalSpace;
   repaint();
  }
 }

 /**
  * Returns the horizontal aligment of the control.
  */
 public int getHorizontalAlignment()
 {
  return horizontalAlignment;
 }

 /**
  * Sets the horizontal aligment of the control, that means whether
  * the part with the label is at the left or right side.<BR>
  * Possible values: haLeftJustify, haRightJustify<BR>
  * If the new value is not 0 or 1 the property is set to
  * haLeftJustify.
  */
 public void setHorizontalAlignment(int newHorizontalAlignment)
 {
  if (horizontalAlignment != newHorizontalAlignment)
  {
   if ((newHorizontalAlignment < 0) || (newHorizontalAlignment > 1))
    horizontalAlignment = haLeftJustify;
   else
    horizontalAlignment = newHorizontalAlignment;
   repaint();
  }
 }

 /**
  * Returns whether the part with the caption is drawn rounded.
  */
 public boolean isRoundLong()
 {
  return roundLong;
 }

 /**
  * Sets whether the part with the caption is drawn rounded.
  */
 public void setRoundLong(boolean newRoundLong)
 {
  if (roundLong != newRoundLong)
  {
   roundLong = newRoundLong;
   repaint();
  }
 }

 /**
  * Returns whether the small part is drawn rounded.
  */
 public boolean isRoundShort()
 {
  return roundShort;
 }

 /**
  * Sets whether the small part is drawn rounded.
  */
 public void setRoundShort(boolean newRoundShort)
 {
  if (roundShort != newRoundShort)
  {
   roundShort = newRoundShort;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the state property is true.
  */
 public Color getCheckedColor()
 {
  return checkedColor;
 }

 /**
  * Sets the color which is used when the state property is true.
  */
 public void setCheckedColor(Color newCheckedColor)
 {
  if (!checkedColor.equals(newCheckedColor))
  {
   checkedColor = newCheckedColor;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the state property is false.
  */
 public Color getUncheckedColor()
 {
  return uncheckedColor;
 }

 /**
  * Sets the color which is used when the state property is false.
  */
 public void setUncheckedColor(Color newUncheckedColor)
 {
  if (!uncheckedColor.equals(newUncheckedColor))
  {
   uncheckedColor = newUncheckedColor;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the control is not enabled.
  */
 public Color getDisabledColor()
 {
  return disabledColor;
 }

 /**
  * Sets the color which is used when the control is not enabled.
  */
 public void setDisabledColor(Color newDisabledColor)
 {
  if (!disabledColor.equals(newDisabledColor))
  {
   disabledColor = newDisabledColor;
   repaint();
  }
 }

 /**
  * Returns the LcarsCheckbox which is associated with this checkbox.
  */
 public LcarsCheckboxGroup getCheckboxGroup()
 {
	return group;
 }

 /**
  * Associates the checkbox with a LcarsCheckboxGroup.<BR>
  * If 2 or more checkboxes are associated with a certain
  * LcarsCheckboxGroup they work as radiobuttons.
  */
 public void setCheckboxGroup(LcarsCheckboxGroup newGroup)
 {
  group = newGroup;
  LcarsCheckboxGroup currentGroup = this.group;
	if (currentGroup != null)
  {
	 currentGroup.setSelectedCheckbox(null);
	}
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseClicked(MouseEvent evt) {}


 /**
  * Event handler (not implemented)
  */
 public void mousePressed(MouseEvent evt) {}

 /**
  * Event handler
  */
 public void mouseReleased(MouseEvent evt)
 {
  if (group != null)
  {
   setState(true);
  }
  else
  {
   Toggle();
  }
  fireAction();
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseEntered(MouseEvent evt) {}

 /**
  * Event handler (not implemented)
  */
 public void mouseExited(MouseEvent evt) {}

 /**
  * Sends the Action-Event to all listeners.
  */
 protected void fireAction()
 {
  ActionEvent actionEvt = new ActionEvent(this, 0, null);
  Vector vec;
  synchronized(this)
  {
   vec = (Vector)listeners.clone();
  }
  for (int i = 0; i < vec.size(); i++)
  {
   ((ActionListener)vec.elementAt(i)).actionPerformed(actionEvt);
  }
 }
 
 /**
  * Adds an Action Listener.
  */
 public synchronized void addActionListener(ActionListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes an Action Listener.
  */
 public synchronized void removeActionListener(ActionListener l)
 {
  listeners.removeElement(l);
 }

}
