
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * An edge (design element).
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsEdge extends Canvas implements Serializable
{
 /**
  * The edge is in the upper left corner.
  */
 public static final int epTopLeft = 0;
 /**
  * The edge is in the upper right corner.
  */
 public static final int epTopRight = 1;
 /**
  * The edge is in the lower right corner.
  */
 public static final int epBottomRight = 2;
 /**
  * The edge is in the lower left corner.
  */
 public static final int epBottomLeft = 3;

 private Image backImage;
 private boolean innerFilled;
 private int edgePosition;
 private int thicknessX;
 private int thicknessY;
 private int innerRadiusX;
 private int innerRadiusY;
 private int outerRadiusX;
 private int outerRadiusY;
 private Vector listeners = new Vector();

 /**
  * Constructs a new instance.
  */
 public LcarsEdge()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  thicknessX = 40;
  thicknessY = 20;
  outerRadiusX = 20;
  outerRadiusY = 20;
  innerRadiusX = 20;
  innerRadiusY = 20;
  setForeground(Color.yellow);
  setSize(100, 100);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {
    this.setSize(new Dimension(427, 318));}

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  int width = getSize().width;
  int height = getSize().height;
  Color foreColor = getForeground();
  Color backColor = getBackground();
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.setColor(backColor);
  backBuffer.fillRect(0, 0, width, height);
  switch (edgePosition)
  {
   case epTopLeft :
    backBuffer.setColor(foreColor);
    backBuffer.fillOval(0, 0, outerRadiusX * 2, outerRadiusY * 2);
    backBuffer.fillRect(outerRadiusX, 0, width, height);
    backBuffer.fillRect(0, outerRadiusY, width, height);
    if (!innerFilled)
    {
     backBuffer.setColor(backColor);
     backBuffer.fillOval(thicknessX, thicknessY, innerRadiusX * 2, innerRadiusY * 2);
     backBuffer.fillRect(thicknessX + innerRadiusX, thicknessY, width, height);
     backBuffer.fillRect(thicknessX, thicknessY + innerRadiusY, width, height);
    }
    break;
   case epTopRight :
    backBuffer.setColor(foreColor);
    backBuffer.fillOval(width - outerRadiusX * 2, 0, outerRadiusX * 2, outerRadiusY * 2);
    backBuffer.fillRect(0, 0, width - outerRadiusX, height);
    backBuffer.fillRect(0, outerRadiusY, width, height);
    if (!innerFilled)
    {
     backBuffer.setColor(backColor);
     backBuffer.fillOval(width - thicknessX - innerRadiusX * 2, thicknessY, innerRadiusX * 2, innerRadiusY * 2);
     backBuffer.fillRect(0, thicknessY, width - thicknessX - innerRadiusX, height);
     backBuffer.fillRect(0, thicknessY + innerRadiusY, width - thicknessX, height);
    }
    break;
   case epBottomRight :
    backBuffer.setColor(foreColor);
    backBuffer.fillOval(width - outerRadiusX * 2, height - outerRadiusY * 2, outerRadiusX * 2, outerRadiusY * 2);
    backBuffer.fillRect(0, 0, width, height - outerRadiusY);
    backBuffer.fillRect(0, 0, width - outerRadiusX, height);
    if (!innerFilled)
    {
     backBuffer.setColor(backColor);
     backBuffer.fillOval(width - thicknessX - innerRadiusX * 2, height - thicknessY - innerRadiusY * 2, innerRadiusX * 2, innerRadiusY * 2);
     backBuffer.fillRect(0, 0, width - thicknessX, height - thicknessY - innerRadiusY);
     backBuffer.fillRect(0, 0, width - thicknessX - innerRadiusX, height - thicknessY);
    }
    break;
   case epBottomLeft :
    backBuffer.setColor(foreColor);
    backBuffer.fillOval(0, height - outerRadiusY * 2, outerRadiusX * 2, outerRadiusY * 2);
    backBuffer.fillRect(0, 0, width, height - outerRadiusY);
    backBuffer.fillRect(outerRadiusX, 0, width, height);
    if (!innerFilled)
    {
    backBuffer.setColor(backColor);
    backBuffer.fillOval(thicknessX, height - thicknessY - innerRadiusY * 2, innerRadiusX * 2, innerRadiusY * 2);
    backBuffer.fillRect(thicknessX, 0, width, height - thicknessY - innerRadiusY);
    backBuffer.fillRect(thicknessX + innerRadiusX, 0, width, height - thicknessY);
    }
    break;
  }
  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Returns the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }

 /**
  * Returns the position of the edge.
  */
 public int getEdgePosition()
 {
  return edgePosition;
 }

 /**
  * Sets the position of the edge.<BR>
  * Possible values: epTopLeft, epTopRight, epBottomRight, epBottomLeft<BR>
  * If the new value is not in the range between 0 and 3 the property is set
  * to epTopLeft.
  */
 public void setEdgePosition(int newEdgePosition)
 {
  if (edgePosition != newEdgePosition)
  {
   if ((newEdgePosition < 0) || (newEdgePosition > 3))
    edgePosition = epTopLeft;
   else
    edgePosition = newEdgePosition;
   repaint();
  }
 }

 /**
  * Returns whether the inner of the edge is drawn using the
  * background color.
  */
 public boolean isInnerFilled()
 {
  return innerFilled;
 }

 /**
  * Sets whether the inner of the edge is drawn using the
  * background color.
  */
 public void setInnerFilled(boolean newInnerFilled)
 {
  if (innerFilled != newInnerFilled)
  {
   innerFilled = newInnerFilled;
   repaint();
  }
 }

 /**
  * Returns the horizontal thickness of the edge.
  */
 public int getThicknessX()
 {
  return thicknessX;
 }

 /**
  * Sets the horizontal thickness of the edge.
  */
 public void setThicknessX(int newThicknessX)
 {
  if (thicknessX != newThicknessX)
  {
   thicknessX = newThicknessX;
   repaint();
  }
 }

 /**
  * Returns the vertical thickness of the edge.
  */
 public int getThicknessY()
 {
  return thicknessY;
 }

 /**
  * Sets the vertical thickness of the edge.
  */
 public void setThicknessY(int newThicknessY)
 {
  if (thicknessY != newThicknessY)
  {
   thicknessY = newThicknessY;
   repaint();
  }
 }

 /**
  * Returns the radius in X direction of the inner side of the edge.
  */
 public int getInnerRadiusX()
 {
  return innerRadiusX;
 }

 /**
  * Sets the radius in X direction of the inner side of the edge.
  */
 public void setInnerRadiusX(int newInnerRadiusX)
 {
  if (innerRadiusX != newInnerRadiusX)
  {
   innerRadiusX = newInnerRadiusX;
   repaint();
  }
 }

 /**
  * Returns the radius in Y direction of the inner side of the edge.
  */
 public int getInnerRadiusY()
 {
  return innerRadiusY;
 }

 /**
  * Sets the radius in Y direction of the inner side of the edge.
  */
 public void setInnerRadiusY(int newInnerRadiusY)
 {
  if (innerRadiusY != newInnerRadiusY)
  {
   innerRadiusY = newInnerRadiusY;
   repaint();
  }
 }

 /**
  * Returns the radius in X direction of the outer side of the edge.
  */
 public int getOuterRadiusX()
 {
  return outerRadiusX;
 }

 /**
  * Sets the radius in X direction of the outer side of the edge.
  */
 public void setOuterRadiusX(int newOuterRadiusX)
 {
  if (outerRadiusX != newOuterRadiusX)
  {
   outerRadiusX = newOuterRadiusX;
   repaint();
  }
 }

 /**
  * Returns the radius in Y direction of the outer side of the edge.
  */
 public int getOuterRadiusY()
 {
  return outerRadiusY;
 }

 /**
  * Sets the radius in Y direction of the outer side of the edge.
  */
 public void setOuterRadiusY(int newOuterRadiusY)
 {
  if (outerRadiusY != newOuterRadiusY)
  {
   outerRadiusY = newOuterRadiusY;
   repaint();
  }
 }

}
