
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A gauge.
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsGauge extends Canvas implements Serializable
{
 private static final int markerThickness = 4;
 private static final int gridStepWidth = 5;
 private static final int spaceBetweenGrids = 2;
 private static final int gridOneMin = 0;
 /**
  * No gridded frame(s) around the gauge bar.
  */
 public static final int gsNormal = 0;
 /**
  * Gridded frame(s) around the gauge bar.
  */
 public static final int gsGridded = 1;
 /**
  * Direction of the gauge: up.
  */
 public static final int gdUp = 0;
 /**
  * Direction of the gauge: right.
  */
 public static final int gdRight = 1;
  /**
  * Direction of the gauge: down.
  */
 public static final int gdDown = 2;
 /**
  * Direction of the gauge: left.
  */
 public static final int gdLeft = 3;
 /**
  * One grid rectangle.
  */
 public static final int ggOne = 1;
 /**
  * Two grid rectangles.
  */
 public static final int ggTwo = 2;
 /**
  * Three grid rectangles.
  */
 public static final int ggThree = 3;
 /**
  * Four grid rectangles.
  */
 public static final int ggFour = 4;

 private Image backImage;
 private Color fontColor;
 private int minValue;
 private int maxValue;
 private int progress;
 private int gaugeStyle;
 private boolean showText;
 private boolean gaugeBounds;
 private int boundsThickness;
 protected Color boundsColor;
 protected Color barColor;
 private int gaugeDirection;
 private int gaugeGrids;
 private int gridTwoAtPercent;
 private int gridThreeAtPercent;
 private int gridFourAtPercent;
 private int zeroPercent;
 private int hundredPercent;
 private Color gridOneColor;
 private Color gridTwoColor;
 private Color gridThreeColor;
 private Color gridFourColor;
 private int gridLineEveryNPercent;
 private int textPlace;
 private int barPlace;
 private int percentPlace;
 private int progressPlace;
 private int progressToShow;
 private int textWidth;
 private int textHeight;
 private int textPosX;
 private int textPosY;
 private int gridTwoMin;
 private int gridThreeMin;
 private int gridFourMin;
 private int gridOneMax;
 private int gridTwoMax;
 private int gridThreeMax;
 private int gridFourMax;
 private Vector listeners = new Vector();

 /**
  * Constructs a new instance.
  */
 public LcarsGauge()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  barColor = Color.cyan;
  boundsColor = Color.red;
  gaugeBounds = false;
  gaugeDirection = gdUp;
  gaugeGrids = 1;
  gaugeStyle = gsNormal;
  gridTwoAtPercent = 40;
  gridThreeAtPercent = 60;
  gridFourAtPercent = 80;
  gridLineEveryNPercent = 10;
  minValue = 0;
  maxValue = 100;
  progress = 0;
  showText = true;
  gridOneColor = Color.green;
  gridTwoColor = Color.yellow;
  gridThreeColor = Color.red;
  gridFourColor = Color.black;
  fontColor = Color.black;
  setSize(50, 200);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {}

 /**
  * Draws the first grid rectangle.
  */
 private void drawGridOne(Graphics backBuffer)
 {
  int counter;
  double percentMax, lineDistance, nextGridLine;
  percentMax = 1.0;
  backBuffer.setColor(gridOneColor);
  if (gaugeGrids == ggOne)
   percentMax = 100.0;
  else
   percentMax = gridTwoAtPercent;
  lineDistance = (gridOneMax - gridOneMin) / (percentMax / gridLineEveryNPercent);
  nextGridLine = gridOneMin;
  if (gaugeDirection == gdRight)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(gridOneMin + counter, counter,
                        gridOneMax - gridOneMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridOneMin;
   while (counter < gridOneMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridOneMax)
    {
     backBuffer.drawLine(counter, boundsThickness,
                         counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdLeft)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(getSize().width - gridOneMax + counter, counter,
                        gridOneMax - gridOneMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridOneMin;
   while (counter < gridOneMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridOneMax)
    {
     backBuffer.drawLine(getSize().width - counter, boundsThickness,
                         getSize().width - counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdUp)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, getSize().height - gridOneMax + counter,
                        getSize().width - 1 - (counter * 2), gridOneMax - gridOneMin - 1 - (counter * 2));
   }
   counter = gridOneMin;
   while (counter < gridOneMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridOneMax)
    {
     backBuffer.drawLine(boundsThickness, getSize().height - counter,
                         getSize().width - 1 - boundsThickness, getSize().height - counter);
    }
   }
  }
  if (gaugeDirection == gdDown)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, gridOneMin + counter,
                        getSize().width - 1 - (counter * 2), gridOneMax - gridOneMin - 1 - (counter * 2));
   }
   counter = gridOneMin;
   while (counter < gridOneMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridOneMax)
    {
     backBuffer.drawLine(boundsThickness, counter,
                         getSize().width - 1 - boundsThickness, counter);
    }
   }
  }
 }

 /**
  * Draws the second grid rectangle.
  */
 private void drawGridTwo(Graphics backBuffer)
 {
  int counter;
  double percentMax, lineDistance, nextGridLine;
  percentMax = 1.0;
  backBuffer.setColor(gridTwoColor);
  if (gaugeGrids == ggTwo)
   percentMax = 100.0;
  else
  {
   if ((gaugeGrids == ggThree) || (gaugeGrids == ggFour))
    percentMax = gridThreeAtPercent;
  }
  lineDistance = (gridTwoMax - gridTwoMin) / ((percentMax - gridTwoAtPercent) / gridLineEveryNPercent);
  nextGridLine = gridTwoMin;
  if (gaugeDirection == gdRight)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(gridTwoMin + counter, counter,
                        gridTwoMax - gridTwoMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridTwoMin;
   while (counter < gridTwoMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridTwoMax)
    {
     backBuffer.drawLine(counter, boundsThickness,
                         counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdLeft)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(getSize().width - gridTwoMax + counter, counter,
                        gridTwoMax - gridTwoMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridTwoMin;
   while (counter < gridTwoMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridTwoMax)
    {
     backBuffer.drawLine(getSize().width - counter, boundsThickness,
                         getSize().width - counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdUp)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, getSize().height - gridTwoMax + counter,
                        getSize().width - 1 - (counter * 2), gridTwoMax - gridTwoMin - 1 - (counter * 2));
   }
   counter = gridTwoMin;
   while (counter < gridTwoMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridTwoMax)
    {
     backBuffer.drawLine(boundsThickness, getSize().height - counter,
                         getSize().width - 1 - boundsThickness, getSize().height - counter);
    }
   }
  }
  if (gaugeDirection == gdDown)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, gridTwoMin + counter,
                        getSize().width - 1 - (counter * 2), gridTwoMax - gridTwoMin - 1 - (counter * 2));
   }
   counter = gridTwoMin;
   while (counter < gridTwoMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridTwoMax)
    {
     backBuffer.drawLine(boundsThickness, counter,
                         getSize().width - 1 - boundsThickness, counter);
    }
   }
  }
 }

 /**
  * Draws the third grid rectangle.
  */
 private void drawGridThree(Graphics backBuffer)
 {
  int counter;
  double percentMax, lineDistance, nextGridLine;
  percentMax = 1.0;
  backBuffer.setColor(gridThreeColor);
  if (gaugeGrids == ggThree)
   percentMax = 100.0;
  else
  {
   if (gaugeGrids == ggFour)
    percentMax = gridFourAtPercent;
  }
  lineDistance = (gridThreeMax - gridThreeMin) / ((percentMax - gridThreeAtPercent) / gridLineEveryNPercent);
  nextGridLine = gridThreeMin;
  if (gaugeDirection == gdRight)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(gridThreeMin + counter, counter,
                        gridThreeMax - gridThreeMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridThreeMin;
   while (counter < gridThreeMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridThreeMax)
    {
     backBuffer.drawLine(counter, boundsThickness,
                         counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdLeft)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(getSize().width - gridThreeMax + counter, counter,
                        gridThreeMax - gridThreeMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridThreeMin;
   while (counter < gridThreeMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridThreeMax)
    {
     backBuffer.drawLine(getSize().width - counter, boundsThickness,
                         getSize().width - counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdUp)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, getSize().height - gridThreeMax + counter,
                        getSize().width - 1 - (counter * 2), gridThreeMax - gridThreeMin - 1 - (counter * 2));
   }
   counter = gridThreeMin;
   while (counter < gridThreeMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridThreeMax)
    {
     backBuffer.drawLine(boundsThickness, getSize().height - counter,
                         getSize().width - 1 - boundsThickness, getSize().height - counter);
    }
   }
  }
  if (gaugeDirection == gdDown)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, gridThreeMin + counter,
                        getSize().width - 1 - (counter * 2), gridThreeMax - gridThreeMin - 1 - (counter * 2));
   }
   counter = gridThreeMin;
   while (counter < gridThreeMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridThreeMax)
    {
     backBuffer.drawLine(boundsThickness, counter,
                         getSize().width - 1 - boundsThickness, counter);
    }
   }
  }
 }

 /**
  * Draws the fourth grid rectangle.
  */
 private void drawGridFour(Graphics backBuffer)
 {
  int counter;
  double percentMax, lineDistance, nextGridLine;
  percentMax = 100.0;
  backBuffer.setColor(gridFourColor);
  lineDistance = (gridFourMax - gridFourMin) / ((percentMax - gridFourAtPercent) / gridLineEveryNPercent);
  nextGridLine = gridFourMin;
  if (gaugeDirection == gdRight)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(gridFourMin + counter, counter,
                        gridFourMax - gridFourMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridFourMin;
   while (counter < gridFourMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridFourMax)
    {
     backBuffer.drawLine(counter, boundsThickness,
                         counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdLeft)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(getSize().width - gridFourMax + counter, counter,
                        gridFourMax - gridFourMin - 1 - (counter * 2), getSize().height - 1 - (counter * 2));
   }
   counter = gridFourMin;
   while (counter < gridFourMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridFourMax)
    {
     backBuffer.drawLine(getSize().width - counter, boundsThickness,
                         getSize().width - counter, getSize().height - 1 - boundsThickness);
    }
   }
  }
  if (gaugeDirection == gdUp)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, getSize().height - gridFourMax + counter,
                        getSize().width - 1 - (counter * 2), gridFourMax - gridFourMin - 1 - (counter * 2));
   }
   counter = gridFourMin;
   while (counter < gridFourMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridFourMax)
    {
     backBuffer.drawLine(boundsThickness, getSize().height - counter,
                         getSize().width - 1 - boundsThickness, getSize().height - counter);
    }
   }
  }
  if (gaugeDirection == gdDown)
  {
   for (counter = 0; counter < boundsThickness; counter++)
   {
    backBuffer.drawRect(counter, gridFourMin + counter,
                        getSize().width - 1 - (counter * 2), gridFourMax - gridFourMin - 1 - (counter * 2));
   }
   counter = gridFourMin;
   while (counter < gridFourMax)
   {
    nextGridLine = nextGridLine + lineDistance;
    counter = (int)nextGridLine;
    if (counter < gridFourMax)
    {
     backBuffer.drawLine(boundsThickness, counter,
                         getSize().width - 1 - boundsThickness, counter);
    }
   }
  }
 }

 /**
  * Calculates the necessary values for drawing the grid rectangles.
  */
 private void drawGridBounds(Graphics backBuffer)
 {
  if (gaugeGrids == ggOne)
  {
   gridOneMax = barPlace;
   drawGridOne(backBuffer);
  }
  if (gaugeGrids == ggTwo)
  {
   gridTwoMax = barPlace;
   gridTwoMin = (int)(barPlace / 100.0 * gridTwoAtPercent);
   gridOneMax = gridTwoMin - spaceBetweenGrids;
   drawGridOne(backBuffer);
   drawGridTwo(backBuffer);
  }
  if (gaugeGrids == ggThree)
  {
   gridThreeMax = barPlace;
   gridThreeMin = (int)(barPlace / 100.0 * gridThreeAtPercent);
   gridTwoMax = gridThreeMin - spaceBetweenGrids;
   gridTwoMin = (int)(barPlace / 100.0 * gridTwoAtPercent);
   gridOneMax = gridTwoMin - spaceBetweenGrids;
   drawGridOne(backBuffer);
   drawGridTwo(backBuffer);
   drawGridThree(backBuffer);
  }
  if (gaugeGrids == ggFour)
  {
   gridFourMax = barPlace;
   gridFourMin = (int)(barPlace / 100.0 * gridFourAtPercent);
   gridThreeMax = gridFourMin - spaceBetweenGrids;
   gridThreeMin = (int)(barPlace / 100.0 * gridThreeAtPercent);
   gridTwoMax = gridThreeMin - spaceBetweenGrids;
   gridTwoMin = (int)(barPlace / 100.0 * gridTwoAtPercent);
   gridOneMax = gridTwoMin - spaceBetweenGrids;
   drawGridOne(backBuffer);
   drawGridTwo(backBuffer);
   drawGridThree(backBuffer);
   drawGridFour(backBuffer);
  }
 }

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  int width = getSize().width;
  int height = getSize().height;
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.clearRect(0, 0, width, height);
  FontMetrics fm = backBuffer.getFontMetrics();
  if (gaugeStyle == gsNormal)
  {
   if (gaugeBounds)
    boundsThickness = 2;
   else
    boundsThickness = 0;
  }
  else
  {
   if (gaugeStyle == gsGridded)
    boundsThickness = 3;
  }
  if (showText)
  {
   if ((gaugeDirection == gdLeft) || (gaugeDirection == gdRight))
    textPlace = fm.stringWidth("88888") + 4;
   else
   {
    if ((gaugeDirection == gdUp) || (gaugeDirection == gdDown))
     textPlace = fm.getAscent() + fm.getDescent() + 4;
   }
   textWidth = fm.stringWidth(String.valueOf(progress));
   textHeight = fm.getAscent() + fm.getDescent();
  }
  else
   textPlace = 0;
  if ((gaugeDirection == gdLeft) || (gaugeDirection == gdRight))
   barPlace = width - textPlace;
  if ((gaugeDirection == gdUp) || (gaugeDirection == gdDown))
   barPlace = height - textPlace;
  progressPlace = maxValue - minValue;
  zeroPercent = boundsThickness + markerThickness;
  hundredPercent = barPlace - boundsThickness;
  percentPlace = hundredPercent - zeroPercent;
  if (gaugeDirection == gdRight)
  {
   calculateNewProgress();
   backBuffer.setColor(barColor);
   backBuffer.fillRect(boundsThickness, boundsThickness,
                       progressToShow + markerThickness, height - (2 * boundsThickness));
   backBuffer.setColor(Color.white);
   backBuffer.fillRect(boundsThickness + progressToShow, boundsThickness,
                       markerThickness, height - (2 * boundsThickness));
   if ((gaugeBounds) && (gaugeStyle == gsNormal))
   {
    backBuffer.setColor(boundsColor);
    backBuffer.drawRect(0, 0, barPlace - 1, height - 1);
    backBuffer.drawRect(1, 1, barPlace - 3, height - 3);
   }
   if (gaugeStyle == gsGridded)
    drawGridBounds(backBuffer);
   if (showText)
   {
    backBuffer.setColor(fontColor);
    textPosX = (width - textPlace) + (textPlace / 2) - (textWidth / 2);
    textPosY = (height + fm.getAscent() - fm.getDescent()) / 2;
    backBuffer.drawString(String.valueOf(progress), textPosX, textPosY);
   }
  }
  if (gaugeDirection == gdLeft)
  {
   calculateNewProgress();
   backBuffer.setColor(barColor);
   backBuffer.fillRect(width - zeroPercent - progressToShow, boundsThickness,
                       progressToShow + markerThickness, height - (2 * boundsThickness));
   backBuffer.setColor(Color.white);
   backBuffer.fillRect(width - zeroPercent - progressToShow, boundsThickness,
                       markerThickness, height - (2 * boundsThickness));
   if ((gaugeBounds) && (gaugeStyle == gsNormal))
   {
    backBuffer.setColor(boundsColor);
    backBuffer.drawRect(width - barPlace, 0, barPlace - 1, height - 1);
    backBuffer.drawRect(width - barPlace + 1, 1, barPlace - 3, height - 3);
   }
   if (gaugeStyle == gsGridded)
    drawGridBounds(backBuffer);
   if (showText)
   {
    backBuffer.setColor(fontColor);
    textPosX = (textPlace / 2) - (textWidth / 2);
    textPosY = (height + fm.getAscent() - fm.getDescent()) / 2;
    backBuffer.drawString(String.valueOf(progress), textPosX, textPosY);
   }
  }
  if (gaugeDirection == gdUp)
  {
   calculateNewProgress();
   backBuffer.setColor(barColor);
   backBuffer.fillRect(boundsThickness, height - zeroPercent - progressToShow,
                       width - (2 * boundsThickness), progressToShow + markerThickness);
   backBuffer.setColor(Color.white);
   backBuffer.fillRect(boundsThickness, height - zeroPercent - progressToShow,
                       width - (2 * boundsThickness), markerThickness);
   if ((gaugeBounds) && (gaugeStyle == gsNormal))
   {
    backBuffer.setColor(boundsColor);
    backBuffer.drawRect(0, height - barPlace, width - 1, barPlace - 1);
    backBuffer.drawRect(1, height - barPlace + 1, width - 3, barPlace - 3);
   }
   if (gaugeStyle == gsGridded)
    drawGridBounds(backBuffer);
   if (showText)
   {
    backBuffer.setColor(fontColor);
    textPosX = (width / 2) - (textWidth / 2);
    textPosY = (textPlace + fm.getAscent() - fm.getDescent()) / 2;
    backBuffer.drawString(String.valueOf(progress), textPosX, textPosY);
   }
  }
  if (gaugeDirection == gdDown)
  {
   calculateNewProgress();
   backBuffer.setColor(barColor);
   backBuffer.fillRect(boundsThickness, boundsThickness,
                       width - (2 * boundsThickness), progressToShow + markerThickness);
   backBuffer.setColor(Color.white);
   backBuffer.fillRect(boundsThickness, boundsThickness + progressToShow,
                       width - (2 * boundsThickness), markerThickness);
   if ((gaugeBounds) && (gaugeStyle == gsNormal))
   {
    backBuffer.setColor(boundsColor);
    backBuffer.drawRect(0, 0, width - 1, barPlace - 1);
    backBuffer.drawRect(1, 1, width - 3, barPlace - 3);
   }
   if (gaugeStyle == gsGridded)
    drawGridBounds(backBuffer);
   if (showText)
   {
    backBuffer.setColor(fontColor);
    textPosX = (width / 2) - (textWidth / 2);
    textPosY = barPlace + ((textPlace + fm.getAscent() - fm.getDescent()) / 2);
    backBuffer.drawString(String.valueOf(progress), textPosX, textPosY);
   }
  }

  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * Converts the value of the progress property into the drawing
  * coordinates (pixel).
  */
 private void calculateNewProgress()
 {
  float faktor;
  faktor = (float)percentPlace / (float)progressPlace;
  progressToShow = (int)(faktor * (progress - minValue));
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Returns the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }

 /**
  * Returns the color used for drawing the text.
  */
 public Color getFontColor()
 {
  return fontColor;
 }

 /**
  * Sets the color used for drawing the text.
  */
 public void setFontColor(Color newFontColor)
 {
  if (!fontColor.equals(newFontColor))
  {
   fontColor = newFontColor;
   repaint();
  }
 }

 /**
  * Returns the style of the gauge.
  */
 public int getGaugeStyle()
 {
  return gaugeStyle;
 }

 /**
  * Sets the style of the gauge.<BR>
  * Possible values: gsNormal, gsGridded<BR>
  * If the new value is not 0 or 1 the property is set to gsNormal.
  */
 public void setGaugeStyle(int newGaugeStyle)
 {
  if (gaugeStyle != newGaugeStyle)
  {
   if ((newGaugeStyle < 0) || (newGaugeStyle > 1))
    gaugeStyle = gsNormal;
   else
    gaugeStyle = newGaugeStyle;
   repaint();
  }
 }

 /**
  * Returns the number of grid rectangles.
  */
 public int getGaugeGrids()
 {
  return gaugeGrids;
 }

 /**
  * Sets the number of grid rectangles.<BR>
  * Possible values: ggOne, ggTwo, ggThree, ggFour<BR>
  * If the new value is not in the range between 0 and 4
  * the property is set to ggOne.
  */
 public void setGaugeGrids(int newGaugeGrids)
 {
  if (gaugeGrids != newGaugeGrids)
  {
   if ((newGaugeGrids < 1) || (newGaugeGrids > 4))
    gaugeGrids = ggOne;
   else
    gaugeGrids = newGaugeGrids;
   repaint();
  }
 }

 /**
  * Returns the percentage at which the second grid rectangle starts.
  */
 public int getGridTwoAtPercent()
 {
  return gridTwoAtPercent;
 }

 /**
 * Sets the percentage at which the second grid rectangle starts.<BR>
 * The values of this property must be multiples of 5 (you can
 * enter whatever you want, it will be rounded automatically).
 */
 public void setGridTwoAtPercent(int newGridTwoAtPercent)
 {
  gridTwoAtPercent = (newGridTwoAtPercent / gridStepWidth) * gridStepWidth;
  repaint();
 }

 /**
  * Returns the percentage at which the third grid rectangle starts.
  */
 public int getGridThreeAtPercent()
 {
  return gridThreeAtPercent;
 }

 /**
 * Sets the percentage at which the third grid rectangle starts.<BR>
 * The values of this property must be multiples of 5 (you can
 * enter whatever you want, it will be rounded automatically).
 */
 public void setGridThreeAtPercent(int newGridThreeAtPercent)
 {
  gridThreeAtPercent = (newGridThreeAtPercent / gridStepWidth) * gridStepWidth;
  repaint();
 }

 /**
  * Returns the percentage at which the fourth grid rectangle starts.
  */
 public int getGridFourAtPercent()
 {
  return gridFourAtPercent;
 }

 /**
 * Sets the percentage at which the fourth grid rectangle starts.<BR>
 * The values of this property must be multiples of 5 (you can
 * enter whatever you want, it will be rounded automatically).
 */
 public void setGridFourAtPercent(int newGridFourAtPercent)
 {
  gridFourAtPercent = (newGridFourAtPercent / gridStepWidth) * gridStepWidth;
  repaint();
 }

 /**
  * Returns whether the current progress is shown beside the bar.
  */
 public boolean isShowText()
 {
  return showText;
 }

 /**
  * Sets whether the current progress is shown beside the bar.
  */
 public void setShowText(boolean newShowText)
 {
  if (showText != newShowText)
  {
   showText = newShowText;
   repaint();
  }
 }

 /**
  * Returns whether there is a rectangle around the bar.
  */
 public boolean isGaugeBounds()
 {
  return gaugeBounds;
 }

 /**
  * Sets whether there is a rectangle around the bar.
  */
 public void setGaugeBounds(boolean newGaugeBounds)
 {
  if (gaugeBounds != newGaugeBounds)
  {
   gaugeBounds = newGaugeBounds;
   repaint();
  }
 }

 /**
  * Returns the color of the rectangle around the bar.
  */
 public Color getBoundsColor()
 {
  return boundsColor;
 }

 /**
  * Sets the color of the rectangle around the bar.
  */
 public void setBoundsColor(Color newBoundsColor)
 {
  if (!boundsColor.equals(newBoundsColor))
  {
   boundsColor = newBoundsColor;
   repaint();
  }
 }

 /**
  * Returns the color of the gauge bar.
  */
 public Color getBarColor()
 {
  return barColor;
 }

 /**
  * Sets the color of the the gauge bar.
  */
 public void setBarColor(Color newBarColor)
 {
  if (!barColor.equals(newBarColor))
  {
   barColor = newBarColor;
   repaint();
  }
 }

 /**
  * Returns the color of the first grid rectangle.
  */
 public Color getGridOneColor()
 {
  return gridOneColor;
 }

 /**
  * Sets the color of the first grid rectangle.
  */
 public void setGridOneColor(Color newGridOneColor)
 {
  if (!gridOneColor.equals(newGridOneColor))
  {
   gridOneColor = newGridOneColor;
   repaint();
  }
 }

 /**
  * Returns the color of the second grid rectangle.
  */
 public Color getGridTwoColor()
 {
  return gridTwoColor;
 }

 /**
  * Sets the color of the second grid rectangle.
  */
 public void setGridTwoColor(Color newGridTwoColor)
 {
  if (!gridTwoColor.equals(newGridTwoColor))
  {
   gridTwoColor = newGridTwoColor;
   repaint();
  }
 }

 /**
  * Returns the color of the third grid rectangle.
  */
 public Color getGridThreeColor()
 {
  return gridThreeColor;
 }

 /**
  * Sets the color of the third grid rectangle.
  */
 public void setGridThreeColor(Color newGridThreeColor)
 {
  if (!gridThreeColor.equals(newGridThreeColor))
  {
   gridThreeColor = newGridThreeColor;
   repaint();
  }
 }

 /**
  * Returns the color of the fourth grid rectangle.
  */
 public Color getGridFourColor()
 {
  return gridFourColor;
 }

 /**
  * Sets the color of the fourth grid rectangle.
  */
 public void setGridFourColor(Color newGridFourColor)
 {
  if (!gridFourColor.equals(newGridFourColor))
  {
   gridFourColor = newGridFourColor;
   repaint();
  }
 }

 /**
  * Returns the direction of the gauge.
  */
 public int getGaugeDirection()
 {
  return gaugeDirection;
 }

 /**
  * Sets the direction of the gauge.<BR>
  * Possible values: gdUp, gdRight, gdDown, gdLeft<BR>
  * If the new value is not in the range between 0 and 3
  * the property is set to gdUp.
  */
 public void setGaugeDirection(int newGaugeDirection)
 {
  if (gaugeDirection != newGaugeDirection)
  {
   if ((newGaugeDirection < 0) || (newGaugeDirection > 3))
    gaugeDirection = 0;
   else
    gaugeDirection = newGaugeDirection;
   repaint();
  }
 }

 /**
  * Returns how much percent place is between the thin lines
  * inside the grids.
  */
 public int getGridLineEveryNPercent()
 {
  return gridLineEveryNPercent;
 }

 /**
  * Sets how much percent place is between the thin lines inside the grids.<BR>
  * Values of this property may be either 5 or 10 (you can enter
  * whatever you want. If the new value is less than 10 the property will be
  * set to 5 otherwise to 10).
  */
 public void setGridLineEveryNPercent(int newGridLineEveryNPercent)
 {
  if (newGridLineEveryNPercent < 10)
   gridLineEveryNPercent = 5;
  else
   gridLineEveryNPercent = 10;
  repaint();
 }

 /**
  * Returns the minimal progress.
  */
 public int getMinValue()
 {
  return minValue;
 }

 /**
  * Sets the minimal progress.<BR>
  * Property should not be less than 0.
  */
 public void setMinValue(int newMinValue)
 {
  if (newMinValue != minValue)
  {
   if (newMinValue > maxValue)
    minValue = progress;
   else
    minValue = newMinValue;
   if (progress < newMinValue)
    progress = newMinValue;
   repaint();
  }
 }

 /**
  * Returns the maximal progress.
  */
 public int getMaxValue()
 {
  return maxValue;
 }

 /**
  * Sets the maximal progress.<BR>
  * Property should not be less than 0.
  */
 public void setMaxValue(int newMaxValue)
 {
  if (newMaxValue != maxValue)
  {
   if (newMaxValue < minValue)
    minValue = progress;
   else
    maxValue = newMaxValue;
   if (progress > newMaxValue)
    progress = newMaxValue;
   repaint();
  }
 }

 /**
  * Returns the current progress.
  */
 public int getProgress()
 {
  return progress;
 }

 /**
  * Sets the current progress.
  */
 public void setProgress(int newProgress)
 {
  if (newProgress < minValue)
   newProgress = minValue;
  if (newProgress > maxValue)
   newProgress = maxValue;
  if (progress != newProgress)
  {
   progress = newProgress;
   fireLcarsGaugeProgressChange();
   repaint();
  }
 }

 /**
  * Adds or subtracts the difference to/from the current progress.
  */
 public void addProgress(int difference)
 {
  setProgress(progress + difference);
 }

 /**
  * Sends the Change-Event to all listeners.
  */
 protected void fireLcarsGaugeProgressChange()
 {
  LcarsGaugeProgressChangedEvent evt = new LcarsGaugeProgressChangedEvent(this);
  Vector vec;
  synchronized(this)
  {
   vec = (Vector)listeners.clone();
  }
  for (int i = 0; i < vec.size(); i++)
  {
   ((LcarsGaugeProgressChangeListener)vec.elementAt(i)).lcarsGaugeProgressChanged(evt);
  }
 }

 /**
  * Adds a listener for the LcarsGaugeProgressChangedEvent.
  */
 public synchronized void addLcarsGaugeProgressChangeListener(LcarsGaugeProgressChangeListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes a listener for the LcarsGaugeProgressChangedEvent.
  */
 public synchronized void removeLcarsGaugeProgressChangeListener(LcarsGaugeProgressChangeListener l)
 {
  listeners.removeElement(l);
 }

 /**
  * Adds an Action Listener.
  */
 public synchronized void addActionListener(ActionListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes an Action Listener.
  */
 public synchronized void removeActionListener(ActionListener l)
 {
  listeners.removeElement(l);
 }

}

