
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;
import java.util.StringTokenizer;

/**
 * A multiline button.
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsMLButton extends Canvas implements Serializable, MouseListener
{
 /**
  * The label is drawn left aligned.
  */
 public static final int taLeft = 0;
 /**
  * The label is drawn centered.
  */
 public static final int taCenter = 1;
 /**
  * The label is drawn right aligned.
  */
 public static final int taRight = 2;

 private Image backImage;
 private String label;
 private String hyphens;
 private Color fontColor;
 private Color enabledColor;
 private Color disabledColor;
 private Color pressedColor;
 private int textAlignment;
 private int labelPosX;
 private int labelPosY;
 private int labelPlace;
 private int labelMarginX;
 private int labelMarginLeft;
 private int labelMarginRight;
 private StringBuffer labelPart;
 private String[] actualLabel;
 private int numLines;
 private int lineCounter;
 private boolean roundLeft;
 private boolean roundRight;
 private int radiusLeft;
 private int radiusRight;
 private boolean mouseIsPressed;
 private Vector listeners = new Vector();

 /**
  * Constructs a new instance.
  */
 public LcarsMLButton()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  roundLeft = true;
  roundRight = true;
  hyphens = " ";
  label = "";
  radiusLeft = getSize().height / 2;
  radiusRight = getSize().height / 2;
  labelMarginX = 5;
  textAlignment = taCenter;
  enabledColor = Color.blue;
  disabledColor = Color.gray;
  mouseIsPressed = false;
  pressedColor = Color.cyan;
  fontColor = Color.black;
  addMouseListener(this);
  setSize(120, 40);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {
    this.setSize(new Dimension(412, 307));}

 /**
  * Splits the label property in fitting parts.
  */
 private String[] prepareLabel(FontMetrics fontInfo)
 {
  lineCounter = 0;
  String[] tempLabel;
  StringTokenizer labelSplitter = new StringTokenizer(label, hyphens, true);
  numLines = labelSplitter.countTokens();
  StringBuffer tempLine = new StringBuffer("");
  StringBuffer leerString = new StringBuffer("");
  tempLabel = new String[numLines];
  labelPart = new StringBuffer("");
  for (lineCounter = 0; lineCounter < numLines; lineCounter++)
  {
   tempLabel[lineCounter] = "";
  }
  lineCounter = 0;
  while (labelSplitter.hasMoreTokens())
  {
   labelPart = null;
   labelPart = new StringBuffer("");
   labelPart.append(labelSplitter.nextToken());
   if ((fontInfo.stringWidth(tempLine.toString()) +
        fontInfo.stringWidth(labelPart.toString())) > labelPlace)
   {
    tempLabel[lineCounter] = tempLine.toString();
    tempLine = new StringBuffer("");
    System.out.println(tempLine.toString());
    lineCounter++;
   }
   tempLine.append(labelPart);
  }
  tempLabel[lineCounter] = tempLine.toString();
  numLines = lineCounter + 1;
  return tempLabel;
 }

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  int lineCounter;
  int maxLineWidth;
  int lineWidth;
  Color foreColor;
  int width = getSize().width;
  int height = getSize().height;
  if (roundLeft == false)
  {
   radiusLeft = 0;
   labelMarginLeft = labelMarginX;
  }
  else
  {
   radiusLeft = height / 2;
   labelMarginLeft = 0;
  }
  if (roundRight == false)
  {
   radiusRight = 0;
   labelMarginRight = labelMarginX;
  }
  else
  {
   radiusRight = height / 2;
   labelMarginRight = 0;
  }
  labelPlace = width - radiusLeft - labelMarginLeft - radiusRight - labelMarginRight;
  if (isEnabled()) foreColor = getEnabledColor();
  else foreColor = getDisabledColor();
  if ((mouseIsPressed) && (isEnabled())) foreColor = getPressedColor();
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.clearRect(0, 0, width, height);
  backBuffer.setColor(foreColor);
  backBuffer.fillRoundRect(0, 0, width, height, height, height);
  if (roundLeft == false)
   backBuffer.fillRect(0, 0, height / 2 + 1, height);
  if (roundRight == false)
   backBuffer.fillRect(width - (height / 2 + 1), 0, width, height);
  backBuffer.setColor(fontColor);
  FontMetrics fm = backBuffer.getFontMetrics();
  actualLabel = new String[numLines];
  if (!label.equals(""))
  {
   actualLabel = prepareLabel(fm);
   lineCounter = 0;
   maxLineWidth = 0;
   labelPosY = fm.getAscent() + (height - numLines * fm.getHeight()) / 2;
   for (lineCounter = 0; lineCounter < numLines; lineCounter++)
   {
    lineWidth = fm.stringWidth(actualLabel[lineCounter]);
    switch(textAlignment)
    {
     case taLeft :
      labelPosX = radiusLeft + labelMarginLeft;
      break;
     case taCenter :
      labelPosX = (width - lineWidth) / 2;
      break;
     case taRight :
      labelPosX = width - radiusRight - labelMarginRight - lineWidth;
      break;
    }
    backBuffer.drawString(actualLabel[lineCounter], labelPosX, labelPosY);
    labelPosY = labelPosY + fm.getHeight();
   }
  }
  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Returns the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }

 /**
  * Returns the color used for drawing the label.
  */
 public Color getFontColor()
 {
  return fontColor;
 }

 /**
  * Sets the color used for drawing the label.
  */
 public void setFontColor(Color newFontColor)
 {
  if (!fontColor.equals(newFontColor))
  {
   fontColor = newFontColor;
   repaint();
  }
 }

 /**
  * Returns the label.
  */
 public String getLabel()
 {
  return label;
 }

 /**
  * Sets the label.
  */
 public void setLabel(String newLabel)
 {
  if (!label.equals(newLabel))
  {
   label = newLabel;
   repaint();
  }
 }

 /**
  * Returns the possible hyphens.
  */
 public String getHyphens()
 {
  return hyphens;
 }

 /**
  * Sets the possible hyphens.<BR>
  * If the label contains a character which is part of the hyphens string
  * a line break will be made if necessary after that character.
  */
 public void setHyphens(String newHyphens)
 {
  if (!hyphens.equals(newHyphens))
  {
   hyphens = newHyphens;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the control is enabled.
  */
 public Color getEnabledColor()
 {
  return enabledColor;
 }

 /**
  * Sets the color which is used when the control is enabled.
  */
 public void setEnabledColor(Color newEnabledColor)
 {
  if (!enabledColor.equals(newEnabledColor))
  {
   enabledColor = newEnabledColor;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the button is pressed.
  */
 public Color getPressedColor()
 {
  return pressedColor;
 }

 /**
  * Sets the color which is used when the button is pressed.
  */
 public void setPressedColor(Color newPressedColor)
 {
  if (!pressedColor.equals(newPressedColor))
  {
   pressedColor = newPressedColor;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the control is not enabled.
  */
 public Color getDisabledColor()
 {
  return disabledColor;
 }

 /**
  * Sets the color which is used when the control is not enabled.
  */
 public void setDisabledColor(Color newDisabledColor)
 {
  if (!disabledColor.equals(newDisabledColor))
  {
   disabledColor = newDisabledColor;
   repaint();
  }
 }

 /**
  * Returns whether the left side of the control is drawn rounded.
  */
 public boolean isRoundLeft()
 {
  return roundLeft;
 }

 /**
  * Sets whether the left side of the control is drawn rounded.
  */
 public void setRoundLeft(boolean newRoundLeft)
 {
  if (roundLeft != newRoundLeft)
  {
   roundLeft = newRoundLeft;
   repaint();
  }
 }

 /**
  * Returns whether the right side of the control is drawn rounded.
  */
 public boolean isRoundRight()
 {
  return roundRight;
 }

 /**
  * Sets whether the right side of the control is drawn rounded.
  */
 public void setRoundRight(boolean newRoundRight)
 {
  if (roundRight != newRoundRight)
  {
   roundRight = newRoundRight;
   repaint();
  }
 }

 /**
  * Returns the horizontal alignment of the label.
  */
 public int getTextAlignment()
 {
  return textAlignment;
 }

 /**
  * Gets the horizontal alignment of the label.<BR>
  * Possible values: taLeft, taCenter, taRight<BR>
  * If the new value is not in the range between 0 and 2 the property is
  * set to taLeft.
  */
 public void setTextAlignment(int newTextAlignment)
 {
  if (textAlignment != newTextAlignment)
  {
   if ((newTextAlignment < 0) || (newTextAlignment > 3))
    textAlignment = taCenter;
   else
    textAlignment = newTextAlignment;
   repaint();
  }
 }

 /**
  * Returns the horizontal margin between the label and the bound of the control.
  */
 public int getLabelMarginX()
 {
  return labelMarginX;
 }

  /**
  * Sets the horizontal margin between the label and the bound of the control
  * if roundLeft or roundRight is set to false.
  */
 public void setLabelMarginX(int newLabelMarginX)
 {
  if (labelMarginX != newLabelMarginX)
  {
   labelMarginX = newLabelMarginX;
   repaint();
  }
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseClicked(MouseEvent evt) {}

 /**
  * Event handler
  */
 public void mousePressed(MouseEvent evt)
 {
  if (isEnabled())
  {
   mouseIsPressed = true;
   if (!pressedColor.equals(enabledColor))
   {
    repaint();
   }
  }
 }

 /**
  * Event handler
  */
 public void mouseReleased(MouseEvent evt)
 {
  mouseIsPressed = false;
  if ((isEnabled()) && (!pressedColor.equals(enabledColor)))
  {
   repaint();
  }
  fireAction();
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseEntered(MouseEvent evt) {}

 /**
  * Event handler (not implemented)
  */
 public void mouseExited(MouseEvent evt) {}

 /**
  * Sends the Action-Event to all listeners.
  */
 protected void fireAction()
 {
  ActionEvent actionEvt = new ActionEvent(this, 0, null);
  Vector vec;
  synchronized(this)
  {
   vec = (Vector)listeners.clone();
  }
  for (int i = 0; i < vec.size(); i++)
  {
   ((ActionListener)vec.elementAt(i)).actionPerformed(actionEvt);
  }
 }

 /**
  * Adds an Action Listener.
  */
 public synchronized void addActionListener(ActionListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes an Action Listener.
  */
 public synchronized void removeActionListener(ActionListener l)
 {
  listeners.removeElement(l);
 }

}
