
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A trackbar.
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsTrackbar extends Canvas
 implements Serializable, MouseListener, MouseMotionListener, LcarsTimerFireListener
{
 private static final int sliderThickness = 6;
 private static final int boundMin = 0;
 private static final int boundsThickness = 2;
 private static final int leftMButton = 16;
 /**
  * Direction of the trackbar: up.
  */
 public static final int tdUp = 0;
 /**
  * Direction of the trackbar: right.
  */
 public static final int tdRight = 1;

 private Image backImage;
 private int minValue;
 private int maxValue;
 private int position;
 private boolean showText;
 private Color trackbarColor;
 private int trackbarDirection;
 private int textFrequency;
 private int tickFrequency;
 private int criticalValue;
 private int zeroPercent;
 private int hundredPercent;
 private int textPlace;
 private int barPlace;
 private int percentPlace;
 private int positionPlace;
 private int positionToShow;
 private int oldToShow;
 private int textWidth;
 private int textHeight;
 private int textPosX;
 private int textPosY;
 private boolean sliderGrabbed;
 private boolean mouseIsPressed;
 private int oldSliderPos;
 private int mEventX;
 private int mEventY;
 private int mEventButton;
 private Vector listeners = new Vector();
 private LcarsTimer sliderTimer;

 /**
  * Constructs a new instance.
  */
 public LcarsTrackbar()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  minValue = 0;
  maxValue = 100;
  criticalValue = 80;
  position = 0;
  textFrequency = 10;
  tickFrequency = 5;
  showText = true;
  trackbarColor = Color.blue;
  trackbarDirection = tdUp;
  addMouseListener(this);
  addMouseMotionListener(this);
  sliderTimer = new LcarsTimer(100, 500);
  sliderTimer.addLcarsTimerFireListener(this);
  setSize(50, 200);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {}

 /**
  * Paints the movable slider.
  */
 private void paintSlider(Graphics backBuffer)
 {
  if (trackbarDirection == tdRight)
  {
   backBuffer.fillRect(zeroPercent + positionToShow - (sliderThickness / 2), boundsThickness,
                       sliderThickness, getSize().height - (2 * boundsThickness));
  }
  if (trackbarDirection == tdUp)
  {
   backBuffer.fillRect(boundsThickness, getSize().height - zeroPercent - positionToShow - (sliderThickness / 2),
                       getSize().width - (2 * boundsThickness), sliderThickness);
  }
 }

 /**
  * Draws the ticks and numbers.
  */
 private void drawTicks(Graphics backBuffer)
 {
  int drawValue, drawPos, count;
  float factor;
  drawValue = minValue;
  FontMetrics fm = backBuffer.getFontMetrics();
  while (drawValue <= maxValue)
  {
   factor = (float)percentPlace / (float)positionPlace;
   drawPos = (int)(factor * (drawValue - minValue));
   if (trackbarDirection == tdRight)
   {
    for (count = -1; count <= 1; count++)
    {
     backBuffer.drawLine(zeroPercent + drawPos + count, boundsThickness,
                         zeroPercent + drawPos + count, boundsThickness + 10);
     backBuffer.drawLine(zeroPercent + drawPos + count, getSize().height - boundsThickness - 10,
                         zeroPercent + drawPos + count, getSize().height - boundsThickness);
    }
    if ((drawValue != minValue) && (drawValue != maxValue))
    {
     backBuffer.drawString(String.valueOf(drawValue),
                           zeroPercent + drawPos - (fm.stringWidth(String.valueOf(drawValue)) / 2),
                           (getSize().height / 2) + ((fm.getAscent() - fm.getDescent()) / 2));
    }
   }
   if (trackbarDirection == tdUp)
   {
    for (count = -1; count <= 1; count++)
    {
     backBuffer.drawLine(boundsThickness, getSize().height - zeroPercent - drawPos + count,
                         boundsThickness + 10, getSize().height - zeroPercent - drawPos + count);
     backBuffer.drawLine(getSize().width - boundsThickness - 10, getSize().height - zeroPercent - drawPos + count,
                         getSize().width - boundsThickness, getSize().height - zeroPercent - drawPos + count);
    }
    if ((drawValue != minValue) && (drawValue != maxValue))
    {
     backBuffer.drawString(String.valueOf(drawValue),
                           (getSize().width / 2) -  (fm.stringWidth(String.valueOf(drawValue)) / 2),
                           getSize().height - zeroPercent - drawPos + ((fm.getAscent() - fm.getDescent()) / 2));
    }
   }
   drawValue = drawValue + textFrequency;
  }
  drawValue = minValue;
  while (drawValue <= maxValue)
  {
   factor = (float)percentPlace / (float)positionPlace;
   drawPos = (int)(factor * (drawValue - minValue));
   if (trackbarDirection == tdRight)
   {
    backBuffer.drawLine(zeroPercent + drawPos, boundsThickness,
                        zeroPercent + drawPos, boundsThickness + 6);
    backBuffer.drawLine(zeroPercent + drawPos, getSize().height - boundsThickness - 6,
                        zeroPercent + drawPos, getSize().height - boundsThickness);
   }
   if (trackbarDirection == tdUp)
   {
    backBuffer.drawLine(boundsThickness, getSize().height - zeroPercent - drawPos,
                        boundsThickness + 6, getSize().height - zeroPercent - drawPos);
    backBuffer.drawLine(getSize().width - boundsThickness - 6, getSize().height - zeroPercent - drawPos,
                        getSize().width - boundsThickness, getSize().height - zeroPercent - drawPos);
   }
   drawValue = drawValue + tickFrequency;
  }
  drawValue = criticalValue;
  factor = (float)percentPlace / (float)positionPlace;
  drawPos = (int)(factor * (drawValue - minValue));
  if (trackbarDirection == tdRight)
  {
   backBuffer.fillRect(zeroPercent + drawPos, boundsThickness,
                       hundredPercent - zeroPercent - drawPos, 4);
   backBuffer.fillRect(zeroPercent + drawPos, getSize().height - boundsThickness - 4,
                       hundredPercent - zeroPercent - drawPos, 4);
  }
  if (trackbarDirection == tdUp)
  {
   backBuffer.fillRect(boundsThickness, getSize().height - hundredPercent,
                       4, hundredPercent - zeroPercent - drawPos);
   backBuffer.fillRect(getSize().width - boundsThickness - 4, getSize().height - hundredPercent,
                       4, hundredPercent - zeroPercent - drawPos);
  }
 }

 /**
  * Calculates the sliders position.
  */
 private void calculateNewPosition()
 {
  float factor;
  oldToShow = positionToShow;
  factor = (float)percentPlace / (float)positionPlace;
  positionToShow = (int)(factor * (position - minValue));
  if (!sliderGrabbed)
  {
   if (trackbarDirection == tdRight)
    oldSliderPos = zeroPercent + positionToShow;
   if (trackbarDirection == tdUp)
    oldSliderPos = getSize().height - zeroPercent - positionToShow;
  }
 }

 /**
  * Calculates the new value of the position property after
  * moving the slider (with the mouse).
  */
 private void updatePosition(int x, int y)
 {
  float factor;
  factor = (float)percentPlace / (float)positionPlace;
  if (trackbarDirection == tdRight)
   positionToShow = x - zeroPercent;
  if (trackbarDirection == tdUp)
   positionToShow = getSize().height - zeroPercent - y;
  if (positionToShow < 0)
   positionToShow = 0;
  if (positionToShow > hundredPercent)
   positionToShow = hundredPercent;
  setPosition((int)(positionToShow / factor) + minValue);
 }

 /**
  * Checks whether the mouse pointer is over the slider.
  */
 private boolean overSlider(int x, int y)
 {
  boolean over = false;
  if (trackbarDirection == tdRight)
  {
   if ((x >= zeroPercent + positionToShow - (sliderThickness / 2)) &&
       (x <= zeroPercent + positionToShow + (sliderThickness / 2)))
    over = true;
  }
  if (trackbarDirection == tdUp)
  {
   if ((y >= getSize().height - zeroPercent - positionToShow - (sliderThickness / 2)) &&
       (y <= getSize().height - zeroPercent - positionToShow + (sliderThickness / 2)))
    over = true;
  }
  return over;
 }

 /**
  * Move the slider to a new position following a mouse event.
  */
 private void moveSlider(int x, int y)
 {
  if (trackbarDirection == tdRight)
  {
   if (x < oldSliderPos)
    addPosition(-1);
   else
   {
    if (x > oldSliderPos)
     addPosition(1);
   }
  }
  if (trackbarDirection == tdUp)
  {
   if (y > oldSliderPos)
    addPosition(-1);
   else
   {
    if (y < oldSliderPos)
     addPosition(1);
   }
  }
 }

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  int width = getSize().width;
  int height = getSize().height;
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.clearRect(0, 0, width, height);
  FontMetrics fm = backBuffer.getFontMetrics();
  if (showText)
  {
   if (trackbarDirection == tdRight)
    textPlace = fm.stringWidth("88888") + 4;
   else
   {
    if (trackbarDirection == tdUp)
     textPlace = fm.getAscent() + fm.getDescent() + 4;
   }
   textWidth = fm.stringWidth(String.valueOf(position));
   textHeight = fm.getAscent() + fm.getDescent();
  }
  else
   textPlace = 0;
  if (trackbarDirection == tdRight)
   barPlace = width - textPlace;
  if (trackbarDirection == tdUp)
   barPlace = height - textPlace;
  positionPlace = maxValue - minValue;
  zeroPercent = boundsThickness + (sliderThickness / 2) + 1;
  hundredPercent = barPlace - boundsThickness - (sliderThickness / 2) - 1;
  percentPlace = hundredPercent - zeroPercent;
  backBuffer.setColor(trackbarColor);
  if (trackbarDirection == tdRight)
  {
   calculateNewPosition();
   drawTicks(backBuffer);
   paintSlider(backBuffer);
   backBuffer.drawRect(0, 0, barPlace - 1, height - 1);
   backBuffer.drawRect(1, 1, barPlace - 3, height - 3);
   if (showText)
   {
    textPosX = (width - textPlace) + (textPlace / 2) - (textWidth / 2);
    textPosY = (height + fm.getAscent() - fm.getDescent()) / 2;
    backBuffer.drawString(String.valueOf(position), textPosX, textPosY);
   }
  }
  if (trackbarDirection == tdUp)
  {
   calculateNewPosition();
   drawTicks(backBuffer);
   paintSlider(backBuffer);
   backBuffer.drawRect(0, height - barPlace, width - 1, barPlace - 1);
   backBuffer.drawRect(1, height - barPlace + 1, width - 3, barPlace - 3);
   if (showText)
   {
    textPosX = (width / 2) - (textWidth / 2);
    textPosY = (textPlace + fm.getAscent() - fm.getDescent()) / 2;
    backBuffer.drawString(String.valueOf(position), textPosX, textPosY);
   }
  }
  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Gets the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }
 
 /**
  * Tells whether the current progress is shown beside the trackbar.
  */
 public boolean isShowText()
 {
  return showText;
 }

 /**
  * Sets whether the current progress is shown beside the trackbar.
  */
 public void setShowText(boolean newShowText)
 {
  if (showText != newShowText)
  {
   showText = newShowText;
   repaint();
  }
 }

 /**
  * Gets the color of the trackbar.
  */
 public Color getTrackbarColor()
 {
  return trackbarColor;
 }

 /**
  * Sets the color of the the trackbar.
  */
 public void setTrackbarColor(Color newTrackbarColor)
 {
  if (!trackbarColor.equals(newTrackbarColor))
  {
   trackbarColor = newTrackbarColor;
   repaint();
  }
 }

 /**
  * Gets the direction of the trackbar.
  */
 public int getTrackbarDirection()
 {
  return trackbarDirection;
 }

 /**
  * Sets the direction of the trackbar.<BR>
  * Possible values: gdUp, gdRight<BR>
  * If the new value is not 0 or 1 the property is set to gdUp.
  */
 public void setTrackbarDirection(int newTrackbarDirection)
 {
  if (trackbarDirection != newTrackbarDirection)
  {
   if ((newTrackbarDirection < 0) || (newTrackbarDirection > 1))
    trackbarDirection = 0;
   else
    trackbarDirection = newTrackbarDirection;
   repaint();
  }
 }

 /**
  * Gets the frequency of the large ticks and numbers.
  */
 public int getTextFrequency()
 {
  return textFrequency;
 }

 /**
  * Sets the frequency of the large ticks and numbers.
  */
 public void setTextFrequency(int newTextFrequency)
 {
  if (newTextFrequency < 1)
   newTextFrequency = 1;
  if (textFrequency != newTextFrequency)
  {
   textFrequency = newTextFrequency;
   repaint();
  }
 }

 /**
  * Gets the frequency of the little ticks.
  */
 public int getTickFrequency()
 {
  return tickFrequency;
 }

 /**
  * Sets the frequency of the little ticks.
  */
 public void setTickFrequency(int newTickFrequency)
 {
  if (newTickFrequency < 1)
   newTickFrequency = 1;
  if (tickFrequency != newTickFrequency)
  {
   tickFrequency = newTickFrequency;
   repaint();
  }
 }

 /**
  * Gets the critical value.
  */
 public int getCriticalValue()
 {
  return criticalValue;
 }

 /**
  * Sets the critical value.<BR>
  * The range between criticalValue and maxValue will be marked. This
  * property has no deeper meaning. If you dont like it, set it to
  * maxValue.
  */
 public void setCriticalValue(int newCriticalValue)
 {
  if (criticalValue != newCriticalValue)
  {
   criticalValue = newCriticalValue;
   repaint();
  }
 }

 /**
  * Gets the minimal position.
  */
 public int getMinValue()
 {
  return minValue;
 }

 /**
  * Sets the minimal position.<BR>
  * Property should not be less than 0.
  */
 public void setMinValue(int newMinValue)
 {
  if (newMinValue != minValue)
  {
   if (newMinValue > maxValue)
    minValue = position;
   else
    minValue = newMinValue;
   if (position < newMinValue)
    position = newMinValue;
   repaint();
  }
 }

 /**
  * Gets the maximal position.
  */
 public int getMaxValue()
 {
  return maxValue;
 }

 /**
  * Sets the maximal position.<BR>
  * Property should not be less than 0.
  */
 public void setMaxValue(int newMaxValue)
 {
  if (newMaxValue != maxValue)
  {
   if (newMaxValue < minValue)
    minValue = position;
   else
    maxValue = newMaxValue;
   if (position > newMaxValue)
    position = newMaxValue;
   repaint();
  }
 }

 /**
  * Gets the current position.
  */
 public int getPosition()
 {
  return position;
 }

 /**
  * Sets the current position.
  */
 public void setPosition(int newPosition)
 {
  if (newPosition < minValue)
   newPosition = minValue;
  if (newPosition > maxValue)
   newPosition = maxValue;
  if (position != newPosition)
  {
   position = newPosition;
   fireLcarsTrackbarPositionChange();
   repaint();
  }
 }

 /**
  * Adds or subtracts the difference to/from the current position.
  */
 public void addPosition(int difference)
 {
  setPosition(position + difference);
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseClicked(MouseEvent evt) {}

 /**
  * Event handler
  */
 public void mousePressed(MouseEvent evt)
 {
  mouseIsPressed = true;
  mEventX = evt.getX();
  mEventY = evt.getY();
  mEventButton = evt.getModifiers();
  if ((mEventButton == leftMButton) && (overSlider(mEventX, mEventY)))
   sliderGrabbed = true;
  if ((mEventButton == leftMButton) && (!overSlider(mEventX, mEventY))
      && (mouseIsPressed))
  {
   if (!sliderTimer.isRunning())
    sliderTimer.start();
   moveSlider(mEventX, mEventY);
  }
 }

 /**
  * Event handler
  */
 public void mouseReleased(MouseEvent evt)
 {
  mouseIsPressed = false;
  mEventButton = evt.getModifiers();
  if ((mEventButton == leftMButton) && (overSlider(mEventX, mEventY)))
   sliderGrabbed = false;
  if (sliderTimer.isRunning())
    sliderTimer.stop();
  fireAction();
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseEntered(MouseEvent evt) {}

 /**
  * Event handler (not implemented)
  */
 public void mouseExited(MouseEvent evt) {}

 /**
  * Event handler
  */
 public void mouseDragged(MouseEvent evt)
 {
  mEventX = evt.getX();
  mEventY = evt.getY();
  mEventButton = evt.getModifiers();
  if (sliderGrabbed)
  {
   updatePosition(mEventX, mEventY);
   if (trackbarDirection == tdRight)
    oldSliderPos = mEventX;
   if (trackbarDirection == tdUp)
    oldSliderPos = mEventY;
  }
  if ((mEventButton == leftMButton) && (overSlider(mEventX, mEventY)))
   sliderGrabbed = true;
  else
   sliderGrabbed = false;
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseMoved(MouseEvent evte) {}

 /**
  * Event handler
  */
 public void lcarsTimerFired(LcarsTimerFiredEvent e)
 {
  if (mouseIsPressed)
  {
   if ((mEventButton == leftMButton) && (overSlider(mEventX, mEventY)))
    sliderGrabbed = true;
   if ((mEventButton == leftMButton) && (!overSlider(mEventX, mEventY))
       && (mouseIsPressed))
    moveSlider(mEventX, mEventY);
  }
 }

 /**
  * Sends the Change-Event to all listeners.
  */
 protected void fireLcarsTrackbarPositionChange()
 {
  LcarsTrackbarPositionChangedEvent evt = new LcarsTrackbarPositionChangedEvent(this);
  Vector vec;
  synchronized(this)
  {
   vec = (Vector)listeners.clone();
  }
  for (int i = 0; i < vec.size(); i++)
  {
   ((LcarsTrackbarPositionChangeListener)vec.elementAt(i)).lcarsTrackbarPositionChanged(evt);
  }
 }

 /**
  * Adds a listener for the LcarsTrackbarPositionChangedEvent.
  */
 public synchronized void addLcarsTrackbarPositionChangeListener(LcarsTrackbarPositionChangeListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes a listener for the LcarsTrackbarPositionChangedEvent.
  */
 public synchronized void removeLcarsTrackbarPositionChangeListener(LcarsTrackbarPositionChangeListener l)
 {
  listeners.removeElement(l);
 }

 /**
  * Sends the Action-Event to all listeners.
  */
 protected void fireAction()
 {
  ActionEvent actionEvt = new ActionEvent(this, 0, null);
  Vector vec;
  synchronized(this)
  {
   vec = (Vector)listeners.clone();
  }
  for (int i = 0; i < vec.size(); i++)
  {
   ((ActionListener)vec.elementAt(i)).actionPerformed(actionEvt);
  }
 }
 
 /**
  * Adds an Action Listener.
  */
 public synchronized void addActionListener(ActionListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes an Action Listener.
  */
 public synchronized void removeActionListener(ActionListener l)
 {
  listeners.removeElement(l);
 }

}